@@ -96,15 +96,6 @@ G_BEGIN_DECLS
   ((MODELINE_SOURCE_XCONFIG)|(MODELINE_SOURCE_NVCONTROL))
 
 
-/* NV-CONTROL metamode sources */
-#define METAMODE_SOURCE_XCONFIG   0x001
-#define METAMODE_SOURCE_IMPLICIT  0x002
-#define METAMODE_SOURCE_NVCONTROL 0x004
-
-#define METAMODE_SOURCE_USER  \
-  ((METAMODE_SOURCE_XCONFIG)|(METAMODE_SOURCE_NVCONTROL))
-
-
 
 
 /*** M A C R O S *************************************************************/
@@ -123,11 +114,42 @@ G_BEGIN_DECLS
 #define GET_MODELINE_HSYNC(m)                                        \
 (((double)((m)->data.clock)) / (2.0f * (double)((m)->data.htotal)))
 
+/* Determines if the metamode was created/modified by the user */
+#define IS_METAMODE_SOURCE_USER(s)      \
+(((s) == METAMODE_SOURCE_XCONFIG) ||    \
+ ((s) == METAMODE_SOURCE_NVCONTROL) ||  \
+ ((s) == METAMODE_SOURCE_RANDR))
 
 
 
 /*** T Y P E   D E F I N I T I O N S *****************************************/
 
+typedef enum {
+    PASSIVE_STEREO_EYE_NONE = 0,
+    PASSIVE_STEREO_EYE_LEFT,
+    PASSIVE_STEREO_EYE_RIGHT,
+} PassiveStereoEye;
+
+typedef enum {
+    ROTATION_0 = 0,
+    ROTATION_90,
+    ROTATION_180,
+    ROTATION_270,
+} Rotation;
+
+typedef enum {
+    REFLECTION_NONE = 0,
+    REFLECTION_X,
+    REFLECTION_Y,
+    REFLECTION_XY,
+} Reflection;
+
+typedef enum {
+    METAMODE_SOURCE_XCONFIG = 0,
+    METAMODE_SOURCE_IMPLICIT,
+    METAMODE_SOURCE_NVCONTROL,
+    METAMODE_SOURCE_RANDR,
+} MetaModeSource;
 
 typedef struct nvModeLineRec {
     struct nvModeLineRec *next;
@@ -144,7 +166,13 @@ typedef struct nvModeLineRec {
 
 
 
-/* Mode (A particular configuration for a display within an X screen) */
+/* Mode (A particular configuration for a display within an X screen)
+ *
+ * NOTE: When metamodes are duplicated, the modes are memcpy'ed over, so
+ *       if new variables are added to the nvModeRec that shouldn't (just)
+ *       be copied, be sure to update this in
+ *       ctk_display_layout_add_screen_metamode().
+ */
 typedef struct nvModeRec {
     struct nvModeRec *next;
 
@@ -161,12 +189,20 @@ typedef struct nvModeRec {
     struct nvModeLineRec *modeline;     /* Modeline this mode references */
     int dummy;                          /* Dummy mode, don't print out */
 
-    int dim[4];                         /* Viewport (absolute) */
+    int viewPortIn[4];                  /* Viewport In (absolute) */
     int pan[4];                         /* Panning Domain (absolute) */
 
+    int viewPortOut[4];                 /* ViewPort Out (WH) */
+
     int position_type;                  /* Relative, Absolute, etc. */
     struct nvDisplayRec *relative_to;   /* Display Relative/RightOf etc */
 
+    PassiveStereoEye passive_stereo_eye; /* Stereo mode 4 per-dpy setting */
+
+
+    Rotation rotation;
+    Reflection reflection;
+
 } nvMode, *nvModePtr;
 
 
@@ -177,11 +213,19 @@ typedef struct nvDisplayRec {
     struct nvDisplayRec *next_in_screen;
     XConfigMonitorPtr    conf_monitor;
 
+    NvCtrlAttributeHandle *handle;      /* NV-CONTROL handle to device */
+
     struct nvGpuRec    *gpu;            /* GPU the display belongs to */
     struct nvScreenRec *screen;         /* X screen the display is tied to */
 
-    unsigned int        device_mask;    /* Bit mask to identify the display */
-    char               *name;           /* Display name (from NV-CONTROL) */
+    char               *logName;        /* Display name (from NV-CONTROL) */
+    char               *typeBaseName;   /* e.g. "CRT", "DFP", "TV" */
+    char               *typeIdName;     /* e.g. "DFP-1", "TV-0" */
+    char               *dpGuidName;     /* e.g. "DP-GUID-11111111-1111-1111-1111-111111111111" */
+    char               *edidHashName;   /* e.g. "DPY-EDID-11111111-1111-1111-1111-111111111111" */
+    char               *targetIdName;   /* e.g. "DPY-3" */
+    char               *randrName;      /* e.g. "VGA-1", "DVI-I-2" */
+
     Bool                is_sdi;         /* Is an SDI display */
 
     nvModeLinePtr       modelines;      /* Modelines validated by X */
@@ -200,7 +244,8 @@ typedef struct nvMetaModeRec {
     struct nvMetaModeRec *next;
 
     int id;     /* Magic id */
-    int source; /* Source of the metamode */
+    int x_idx;  /* Used to re-order metamodes on apply */
+    MetaModeSource source; /* Source of the metamode */
     Bool switchable; /* Can the metamode be accessed through Ctrl Alt +- */
 
     // Used for drawing & moving metamode boxes
@@ -238,8 +283,9 @@ typedef struct nvScreenRec {
     struct nvGpuRec *gpu;  /* GPU driving this X screen */
 
     int depth;      /* Depth of the screen */
+    int stereo;     /* Stereo mode enabled on this screen */
+    int overlay;    /* Overlay enabled on this screen */
 
-    unsigned int displays_mask; /* Display devices on this X screen */
     nvDisplayPtr displays; /* List of displays using this screen */
     int num_displays; /* # of displays using this screen */
 
@@ -259,6 +305,7 @@ typedef struct nvScreenRec {
     Bool sli;
     Bool dynamic_twinview;  /* This screen supports dynamic twinview */
     Bool no_scanout;        /* This screen has no display devices */
+    Bool stereo_supported;  /* Can stereo be configured on this screen */
 
 } nvScreen, *nvScreenPtr;
 
@@ -289,13 +336,15 @@ typedef struct nvGpuRec {
 
     char *name;  /* Name of the GPU */
 
-    unsigned int connected_displays;  /* Bitmask of connected displays */
-
     gchar *pci_bus_id;
 
     GvoModeData *gvo_mode_data; /* Information about GVO modes available */
     unsigned int num_gvo_modes;
 
+    unsigned int *flags_memory; /* Pointer to memory alloced for flags */
+    unsigned int *flags; /* Array of flags queried from the X server */
+    int num_flags;
+
     nvDisplayPtr displays; /* Linked list of displays connected to GPU */
     int num_displays;
 
@@ -477,6 +526,14 @@ void ctk_display_layout_set_mode_modeline (CtkDisplayLayout *,
                                            nvModePtr mode,
                                            nvModeLinePtr modeline);
 
+void ctk_display_layout_set_mode_viewport_in(CtkDisplayLayout *ctk_object,
+                                             nvModePtr mode,
+                                             int w, int h);
+
+void ctk_display_layout_set_mode_viewport_out(CtkDisplayLayout *ctk_object,
+                                              nvModePtr mode,
+                                              int x, int y, int w, int h);
+
 void ctk_display_layout_set_display_position (CtkDisplayLayout *ctk_object,
                                               nvDisplayPtr display,
                                               int position_type,
@@ -485,8 +542,19 @@ void ctk_display_layout_set_display_position (CtkDisplayLayout *ctk_object,
 void ctk_display_layout_set_display_panning (CtkDisplayLayout *ctk_object,
                                              nvDisplayPtr display,
                                              int width, int height);
+
+void ctk_display_layout_set_display_rotation (CtkDisplayLayout *ctk_object,
+                                              nvDisplayPtr display,
+                                              Rotation rotation);
+
+void ctk_display_layout_set_display_reflection (CtkDisplayLayout *ctk_object,
+                                                nvDisplayPtr display,
+                                                Reflection reflection);
+
 void ctk_display_layout_select_display (CtkDisplayLayout *ctk_object,
                                         nvDisplayPtr display);
+void ctk_display_layout_select_screen (CtkDisplayLayout *ctk_object,
+                                       nvScreenPtr screen);
 void ctk_display_layout_update_display_count (CtkDisplayLayout *,
                                               nvDisplayPtr);
 
